import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, min_os_level

MIODeviceStreamQueueAlteredProc = b"vI^v^v"

CMIOStreamScheduledOutputNotificationProc = b"vQQ^v"


class TestCMIOSampleBuffer(TestCase):
    def testDefines(self):
        self.assertEqual(CoreMediaIO.CMIOGetNextSequenceNumber(0), 1)
        self.assertEqual(CoreMediaIO.CMIOGetNextSequenceNumber(42), 43)
        self.assertEqual(
            CoreMediaIO.CMIOGetNextSequenceNumber(
                CoreMediaIO.kCMIOInvalidSequenceNumber
            ),
            0,
        )

        self.assertIs(
            CoreMediaIO.CMIODiscontinuityFlagsHaveHardDiscontinuities(
                CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_MalformedData
                | CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_DurationWasExtended
            ),
            True,
        )
        self.assertIs(
            CoreMediaIO.CMIODiscontinuityFlagsHaveHardDiscontinuities(
                CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_MalformedData
            ),
            False,
        )

    def testConstants(self):
        self.assertEqual(CoreMediaIO.kCMIOInvalidSequenceNumber, 0xFFFFFFFFFFFFFFFF)

        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDiscontinuities, 0)

        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_UnknownDiscontinuity, 1 << 0
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_TimecodeDiscontinuity, 1 << 1
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_PacketError, 1 << 2
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_StreamDiscontinuity, 1 << 3
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_MalformedData, 1 << 4
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_DataWasFlushed, 1 << 5
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_DataWasDropped, 1 << 6
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_BufferOverrun, 1 << 7
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_DiscontinuityInDTS, 1 << 8
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_RelatedToDiscontinuity,
            1 << 9,
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_ClientSyncDiscontinuity,
            1 << 10,
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_TrickPlay, 1 << 11
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_NoDataMarker, 1 << 12
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_DataFormatChanged, 1 << 13
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_TimingReferenceJumped,
            1 << 14,
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_DurationWasExtended, 1 << 15
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_SleepWakeCycle, 1 << 16
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSampleBufferDiscontinuityFlag_CodecSettingsChanged, 1 << 17
        )

        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDataEvent_Unknown, 0)
        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDataEvent_NoMedia, 1)
        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDataEvent_DeviceDidNotSync, 2)
        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDataEvent_DeviceInWrongMode, 3)
        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDataEvent_ProcessingError, 4)
        self.assertEqual(CoreMediaIO.kCMIOSampleBufferNoDataEvent_SleepWakeCycle, 5)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_DiscontinuityFlags, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_SequenceNumber, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_HDV1_PackData, str
        )
        self.assertIsInstance(CoreMediaIO.kCMIOSampleBufferAttachmentKey_HDV2_VAUX, str)
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_CAAudioTimeStamp, str
        )
        self.assertIsInstance(CoreMediaIO.kCMIOSampleBufferAttachmentKey_SMPTETime, str)
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_NativeSMPTEFrameCount, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_NumberOfVideoFramesInBuffer, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_NumberOfVideoFramesInGOP, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_MuxedSourcePresentationTimeStamp,
            str,
        )
        self.assertIsInstance(CoreMediaIO.kCMIOSampleBufferAttachmentKey_HostTime, str)
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_RepeatedBufferContents, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_SourceAudioFormatDescription, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_PulldownCadenceInfo, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_ClosedCaptionSampleBuffer, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_NoDataMarker, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOBlockBufferAttachmentKey_CVPixelBufferReference, str
        )

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_ClientSequenceID, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachmentKey_MouseAndKeyboardModifiers, str
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorPositionX,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorPositionY,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_MouseButtonState,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_KeyboardModifiers,
            str,
        )

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorIsVisible,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorFrameRect,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorReference,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorSeed,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorScale,
            str,
        )

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorIsDrawnInFramebuffer,
            str,
        )
        self.assertIsInstance(
            CoreMediaIO.kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_KeyboardModifiersEvent,
            str,
        )

    def testFunctions(self):
        self.assertArgIsIn(CoreMediaIO.CMIOSampleBufferCreate, 5)
        self.assertArgSizeInArg(CoreMediaIO.CMIOSampleBufferCreate, 5, 4)
        self.assertArgIsIn(CoreMediaIO.CMIOSampleBufferCreate, 7)
        self.assertArgSizeInArg(CoreMediaIO.CMIOSampleBufferCreate, 7, 6)
        self.assertArgIsOut(CoreMediaIO.CMIOSampleBufferCreate, 10)
        self.assertArgIsCFRetained(CoreMediaIO.CMIOSampleBufferCreate, 10)

        self.assertArgIsIn(CoreMediaIO.CMIOSampleBufferCreateForImageBuffer, 3)
        self.assertArgIsOut(CoreMediaIO.CMIOSampleBufferCreateForImageBuffer, 6)
        self.assertArgIsCFRetained(CoreMediaIO.CMIOSampleBufferCreateForImageBuffer, 6)

        self.assertArgIsOut(CoreMediaIO.CMIOSampleBufferCreateNoDataMarker, 5)
        self.assertArgIsCFRetained(CoreMediaIO.CMIOSampleBufferCreateNoDataMarker, 5)

        CoreMediaIO.CMIOSampleBufferSetSequenceNumber
        CoreMediaIO.CMIOSampleBufferGetSequenceNumber
        CoreMediaIO.CMIOSampleBufferSetDiscontinuityFlags
        CoreMediaIO.CMIOSampleBufferGetDiscontinuityFlags
        CoreMediaIO.CMIOSampleBufferCopyNonRequiredAttachments
        CoreMediaIO.CMIOSampleBufferCopySampleAttachments



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, fourcc

CMIOObjectPropertyListenerProc = b"iII^{CMIOObjectPropertyAddress=III}^v"
CMIOObjectPropertyListenerBlock = b"vIn^{CMIOObjectPropertyAddress=III}"


class TestCMIOHardwareObject(TestCase):
    def testStructs(self):
        v = CoreMediaIO.CMIOObjectPropertyAddress()
        self.assertEqual(v.mSelector, 0)
        self.assertEqual(v.mScope, 0)
        self.assertEqual(v.mElement, 0)
        self.assertPickleRoundTrips(v)

    def testConstants(self):
        self.assertEqual(
            CoreMediaIO.kCMIOObjectPropertySelectorWildcard, fourcc(b"****")
        )
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyScopeWildcard, fourcc(b"****"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyElementWildcard, 0xFFFFFFFF)

        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyScopeGlobal, fourcc(b"glob"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyElementMaster, 0)
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyElementMain, 0)
        self.assertEqual(CoreMediaIO.kCMIOObjectClassID, fourcc(b"aobj"))
        self.assertEqual(CoreMediaIO.kCMIOObjectClassIDWildcard, fourcc(b"****"))
        self.assertEqual(CoreMediaIO.kCMIOObjectUnknown, 0)

        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyClass, fourcc(b"clas"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyOwner, fourcc(b"stdv"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyCreator, fourcc(b"oplg"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyName, fourcc(b"lnam"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyManufacturer, fourcc(b"lmak"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyElementName, fourcc(b"lchn"))
        self.assertEqual(
            CoreMediaIO.kCMIOObjectPropertyElementCategoryName, fourcc(b"lccn")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOObjectPropertyElementNumberName, fourcc(b"lcnn")
        )
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyOwnedObjects, fourcc(b"ownd"))
        self.assertEqual(CoreMediaIO.kCMIOObjectPropertyListenerAdded, fourcc(b"lisa"))
        self.assertEqual(
            CoreMediaIO.kCMIOObjectPropertyListenerRemoved, fourcc(b"lisr")
        )

    def testFunctions(self):
        CoreMediaIO.CMIOObjectShow
        self.assertArgIsIn(CoreMediaIO.CMIOObjectHasProperty, 1)

        self.assertArgIsIn(CoreMediaIO.CMIOObjectIsPropertySettable, 1)
        self.assertArgIsOut(CoreMediaIO.CMIOObjectIsPropertySettable, 2)

        self.assertArgIsIn(CoreMediaIO.CMIOObjectGetPropertyDataSize, 1)
        self.assertArgIsIn(CoreMediaIO.CMIOObjectGetPropertyDataSize, 3)
        self.assertArgSizeInArg(CoreMediaIO.CMIOObjectGetPropertyDataSize, 3, 2)
        self.assertArgIsOut(CoreMediaIO.CMIOObjectGetPropertyDataSize, 4)

        self.assertArgIsIn(CoreMediaIO.CMIOObjectGetPropertyData, 1)
        self.assertArgIsOut(CoreMediaIO.CMIOObjectGetPropertyData, 3)
        self.assertArgSizeInArg(CoreMediaIO.CMIOObjectGetPropertyData, 3, 2)
        self.assertArgIsOut(CoreMediaIO.CMIOObjectGetPropertyData, 5)
        self.assertArgIsOut(CoreMediaIO.CMIOObjectGetPropertyData, 6)
        self.assertArgSizeInArg(CoreMediaIO.CMIOObjectGetPropertyData, 6, (4, 5))

        self.assertArgIsIn(CoreMediaIO.CMIOObjectAddPropertyListener, 1)
        self.assertArgIsFunction(
            CoreMediaIO.CMIOObjectAddPropertyListener,
            2,
            CMIOObjectPropertyListenerProc,
            True,
        )

        self.assertArgIsIn(CoreMediaIO.CMIOObjectRemovePropertyListener, 1)
        self.assertArgIsFunction(
            CoreMediaIO.CMIOObjectRemovePropertyListener,
            2,
            CMIOObjectPropertyListenerProc,
            True,
        )

        self.assertArgIsIn(CoreMediaIO.CMIOObjectAddPropertyListenerBlock, 1)
        self.assertArgIsBlock(
            CoreMediaIO.CMIOObjectAddPropertyListenerBlock,
            3,
            CMIOObjectPropertyListenerBlock,
        )

        self.assertArgIsIn(CoreMediaIO.CMIOObjectRemovePropertyListenerBlock, 1)
        self.assertArgIsBlock(
            CoreMediaIO.CMIOObjectRemovePropertyListenerBlock,
            3,
            CMIOObjectPropertyListenerBlock,
        )



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, min_sdk_level, min_os_level


class TestCMIOExtensionProviderHelper(CoreMediaIO.NSObject):
    def connectClient_error_(self, a, b):
        return 1

    def providerPropertiesForProperties_error_(self, a, b):
        return 1

    def setProviderProperties_error_(self, a, b):
        return 1


class TestCMIOExtensionProvider(TestCase):
    @min_os_level("12.3")
    def test_constants(self):
        self.assertIsInstance(CoreMediaIO.CMIOExtensionInfoDictionaryKey, str)
        self.assertIsInstance(CoreMediaIO.CMIOExtensionMachServiceNameKey, str)

    @min_sdk_level("12.3")
    def test_protocols(self):
        self.assertProtocolExists("CMIOExtensionProviderSource")

    def test_methods(self):
        self.assertResultIsBOOL(TestCMIOExtensionProviderHelper.connectClient_error_)
        self.assertArgHasType(
            TestCMIOExtensionProviderHelper.connectClient_error_, 1, b"o^@"
        )

        self.assertArgHasType(
            TestCMIOExtensionProviderHelper.providerPropertiesForProperties_error_,
            1,
            b"o^@",
        )

        self.assertResultIsBOOL(
            TestCMIOExtensionProviderHelper.setProviderProperties_error_
        )
        self.assertArgHasType(
            TestCMIOExtensionProviderHelper.setProviderProperties_error_, 1, b"o^@"
        )

    @min_os_level("12.3")
    def test_methods12_3(self):
        self.assertResultIsBOOL(CoreMediaIO.CMIOExtensionProvider.addDevice_error_)
        self.assertArgIsOut(CoreMediaIO.CMIOExtensionProvider.addDevice_error_, 1)

        self.assertResultIsBOOL(CoreMediaIO.CMIOExtensionProvider.removeDevice_error_)
        self.assertArgIsOut(CoreMediaIO.CMIOExtensionProvider.removeDevice_error_, 1)



"""
Python mapping for the CoreMediaIO framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import sys

import Foundation
import objc
from CoreMediaIO import _CoreMediaIO, _metadata

sys.modules["CoreMediaIO"] = mod = objc.ObjCLazyModule(
    "CoreMediaIO",
    "com.apple.CoreMediaIO",
    objc.pathForFramework("/System/Library/Frameworks/CoreMediaIO.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (Foundation,),
)


del sys.modules["CoreMediaIO._metadata"]


for nm in dir(_CoreMediaIO):
    setattr(mod, nm, getattr(_CoreMediaIO, nm))


def CMIOGetNextSequenceNumber(value):
    if value == 0xFFFFFFFFFFFFFFFF:
        return 0
    return value + 1


mod.CMIOGetNextSequenceNumber = CMIOGetNextSequenceNumber


def CMIODiscontinuityFlagsHaveHardDiscontinuities(value):
    return (value & mod.kCMIOSampleBufferDiscontinuityFlag_DurationWasExtended) != 0


mod.CMIODiscontinuityFlagsHaveHardDiscontinuities = (
    CMIODiscontinuityFlagsHaveHardDiscontinuities
)



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, min_sdk_level, min_os_level


class TestCMIOExtensionDeviceHelper(CoreMediaIO.NSObject):
    def devicePropertiesForProperties_error_(self, a, b):
        return 1

    def setDeviceProperties_error_(self, a, b):
        return 1


class TestCMIOExtensionDevice(TestCase):
    @min_sdk_level("12.3")
    def test_protocols(self):
        self.assertProtocolExists("CMIOExtensionDeviceSource")

    def test_methods(self):
        self.assertArgHasType(
            TestCMIOExtensionDeviceHelper.devicePropertiesForProperties_error_,
            1,
            b"o^@",
        )

        self.assertResultIsBOOL(
            TestCMIOExtensionDeviceHelper.setDeviceProperties_error_
        )
        self.assertArgHasType(
            TestCMIOExtensionDeviceHelper.setDeviceProperties_error_, 1, b"o^@"
        )

    @min_os_level("12.3")
    def test_methods12_3(self):
        self.assertResultIsBOOL(CoreMediaIO.CMIOExtensionDevice.addStream_error_)
        self.assertArgIsOut(CoreMediaIO.CMIOExtensionDevice.addStream_error_, 1)

        self.assertResultIsBOOL(CoreMediaIO.CMIOExtensionDevice.removeStream_error_)
        self.assertArgIsOut(CoreMediaIO.CMIOExtensionDevice.removeStream_error_, 1)



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, fourcc

MIODeviceStreamQueueAlteredProc = b"vI^v^v"

CMIOStreamScheduledOutputNotificationProc = b"vQQ^v"


class TestCMIOHardwareStream(TestCase):
    def testConstants(self):
        self.assertEqual(CoreMediaIO.kCMIOStreamClassID, fourcc(b"astr"))
        self.assertEqual(CoreMediaIO.kCMIOStreamUnknown, CoreMediaIO.kCMIOObjectUnknown)

        self.assertEqual(CoreMediaIO.kCMIODeckStatusBusy, 1)
        self.assertEqual(CoreMediaIO.kCMIODeckStatusLocal, 2)
        self.assertEqual(CoreMediaIO.kCMIODeckStatusNotThreaded, 3)
        self.assertEqual(CoreMediaIO.kCMIODeckStatusTapeInserted, 4)
        self.assertEqual(CoreMediaIO.kCMIODeckStatusOpcode, 5)
        self.assertEqual(CoreMediaIO.kCMIODeckStatusSearchingForDevice, 6)
        self.assertEqual(CoreMediaIO.kCMIODeckStatusNoDevice, 7)

        self.assertEqual(CoreMediaIO.kCMIODeckStateStop, 0)
        self.assertEqual(CoreMediaIO.kCMIODeckStatePlay, 1)
        self.assertEqual(CoreMediaIO.kCMIODeckStatePause, 2)
        self.assertEqual(CoreMediaIO.kCMIODeckStatePlaySlow, 3)
        self.assertEqual(CoreMediaIO.kCMIODeckStateReverseSlow, 4)
        self.assertEqual(CoreMediaIO.kCMIODeckStatePlayReverse, 5)
        self.assertEqual(CoreMediaIO.kCMIODeckStateFastForward, 6)
        self.assertEqual(CoreMediaIO.kCMIODeckStateFastRewind, 7)

        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseHighSpeed, -10)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseFastest, -9)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseFaster, -8)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseFast, -7)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverse1x, -6)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseSlow3, -5)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseSlow2, -4)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseSlow1, -3)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttleReverseSlowest, -2)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlayPreviousFrame, -1)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePause, 0)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlayNextFrame, 1)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlaySlowest, 2)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlaySlow1, 3)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlaySlow2, 4)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlaySlow3, 5)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlay1x, 6)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlayFast, 7)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlayFaster, 8)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlayFastest, 9)
        self.assertEqual(CoreMediaIO.kCMIODeckShuttlePlayHighSpeed, 10)

        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyDirection, fourcc(b"sdir"))
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyTerminalType, fourcc(b"term"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyStartingChannel, fourcc(b"schn")
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyLatency, fourcc(b"ltnc"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyFormatDescription, fourcc(b"pft ")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyFormatDescriptions, fourcc(b"pfta")
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyStillImage, fourcc(b"stmg"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyStillImageFormatDescriptions, fourcc(b"stft")
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyFrameRate, fourcc(b"nfrt"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyMinimumFrameRate, fourcc(b"mfrt")
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyFrameRates, fourcc(b"nfr#"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyFrameRateRanges, fourcc(b"frrg")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyNoDataTimeoutInMSec, fourcc(b"pmn1")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyDeviceSyncTimeoutInMSec, fourcc(b"pmn2")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyNoDataEventCount, fourcc(b"pmn3")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyOutputBufferUnderrunCount, fourcc(b"pmou")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyOutputBufferRepeatCount, fourcc(b"pmor")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyOutputBufferQueueSize, fourcc(b"pmoq")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyOutputBuffersRequiredForStartup,
            fourcc(b"pmos"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyOutputBuffersNeededForThrottledPlayback,
            fourcc(b"miff"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyFirstOutputPresentationTimeStamp,
            fourcc(b"popt"),
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyEndOfData, fourcc(b"pmed"))
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyClock, fourcc(b"pmcl"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyCanProcessDeckCommand, fourcc(b"pdcd")
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyDeck, fourcc(b"deck"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyDeckFrameNumber, fourcc(b"tcod")
        )
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyDeckDropness, fourcc(b"drop"))
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyDeckThreaded, fourcc(b"thrd"))
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyDeckLocal, fourcc(b"locl"))
        self.assertEqual(CoreMediaIO.kCMIOStreamPropertyDeckCueing, fourcc(b"cuec"))
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyInitialPresentationTimeStampForLinkedAndSyncedAudio,
            fourcc(b"ipls"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyScheduledOutputNotificationProc,
            fourcc(b"sonp"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyPreferredFormatDescription, fourcc(b"prfd")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOStreamPropertyPreferredFrameRate, fourcc(b"prfr")
        )

    def testStructs(self):
        v = CoreMediaIO.CMIOStreamDeck()
        self.assertEqual(v.mStatus, 0)
        self.assertEqual(v.mState, 0)
        self.assertEqual(v.mState2, 0)
        self.assertPickleRoundTrips(v)

        v = CoreMediaIO.CMIOStreamScheduledOutputNotificationProcAndRefCon()
        self.assertEqual(v.scheduledOutputNotificationProc, None)
        self.assertEqual(v.scheduledOutputNotificationRefCon, None)
        self.assertPickleRoundTrips(v)

    def testFunctions(self):
        self.assertArgIsOut(CoreMediaIO.CMIOStreamCopyBufferQueue, 3)

        CoreMediaIO.CMIOStreamDeckPlay
        CoreMediaIO.CMIOStreamDeckStop
        CoreMediaIO.CMIOStreamDeckJog

        self.assertArgIsBOOL(CoreMediaIO.CMIOStreamDeckCueTo, 2)

        self.assertArgIsOut(CoreMediaIO.CMIOStreamClockCreate, 6)
        self.assertArgIsCFRetained(CoreMediaIO.CMIOStreamClockCreate, 6)

        CoreMediaIO.CMIOStreamClockPostTimingEvent
        CoreMediaIO.CMIOStreamClockInvalidate
        CoreMediaIO.CMIOStreamClockConvertHostTimeToDeviceTime



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, fourcc


class TestCMIOHardware(TestCase):
    def testConstants(self):
        self.assertEqual(CoreMediaIO.kCMIOHardwareNoError, 0)
        self.assertEqual(CoreMediaIO.kCMIOHardwareNotStoppedError, fourcc(b"run "))
        self.assertEqual(CoreMediaIO.kCMIOHardwareNotRunningError, fourcc(b"stop"))
        self.assertEqual(CoreMediaIO.kCMIOHardwareUnspecifiedError, fourcc(b"what"))
        self.assertEqual(CoreMediaIO.kCMIOHardwareUnknownPropertyError, fourcc(b"who?"))
        self.assertEqual(CoreMediaIO.kCMIOHardwareBadPropertySizeError, fourcc(b"!siz"))
        self.assertEqual(
            CoreMediaIO.kCMIOHardwareIllegalOperationError, fourcc(b"nope")
        )
        self.assertEqual(CoreMediaIO.kCMIOHardwareBadObjectError, fourcc(b"!obj"))
        self.assertEqual(CoreMediaIO.kCMIOHardwareBadDeviceError, fourcc(b"!dev"))
        self.assertEqual(CoreMediaIO.kCMIOHardwareBadStreamError, fourcc(b"!str"))
        self.assertEqual(
            CoreMediaIO.kCMIOHardwareUnsupportedOperationError, fourcc(b"unop")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwareSuspendedBySystemError, fourcc(b"deny")
        )
        self.assertEqual(CoreMediaIO.kCMIODeviceUnsupportedFormatError, fourcc(b"!dat"))
        self.assertEqual(CoreMediaIO.kCMIODevicePermissionsError, fourcc(b"!hog"))

        self.assertEqual(CoreMediaIO.kCMIOPlugInClassID, fourcc(b"aplg"))

        self.assertEqual(CoreMediaIO.kCMIOPlugInPropertyBundleID, fourcc(b"piid"))
        self.assertEqual(CoreMediaIO.kCMIOPlugInPropertyIsExtension, fourcc(b"piie"))



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, min_os_level, fourcc
import objc

CMIODeviceGetSMPTETimeProc = b"d^v^Q^Z^I"


class TestCMIOHardwareDevice(TestCase):
    def testStructs(self):
        v = CoreMediaIO.CMIODeviceStreamConfiguration()
        self.assertEqual(v.mNumberStreams, 0)
        self.assertEqual(v.mNumberChannels, None)
        self.assertPickleRoundTrips(v)

        v = CoreMediaIO.CMIODeviceAVCCommand()
        self.assertEqual(v.mCommand, None)
        self.assertEqual(v.mCommandLength, 0)
        self.assertEqual(v.mResponse, None)
        self.assertEqual(v.mResponseLength, 0)
        self.assertEqual(v.mResponseUsed, 0)
        self.assertPickleRoundTrips(v)

        v = CoreMediaIO.CMIODeviceRS422Command()
        self.assertEqual(v.mCommand, None)
        self.assertEqual(v.mCommandLength, 0)
        self.assertEqual(v.mResponse, None)
        self.assertEqual(v.mResponseLength, 0)
        self.assertEqual(v.mResponseUsed, 0)
        self.assertPickleRoundTrips(v)

        v = CoreMediaIO.CMIODeviceSMPTETimeCallback()
        self.assertEqual(v.mGetSMPTETimeProc, None)
        self.assertEqual(v.mRefCon, None)
        self.assertPickleRoundTrips(v)

    def testConstants(self):
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyScopeInput, fourcc(b"inpt"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyScopeOutput, fourcc(b"outp"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyScopePlayThrough, fourcc(b"ptru")
        )
        self.assertEqual(CoreMediaIO.kCMIODeviceClassID, fourcc(b"adev"))
        self.assertEqual(CoreMediaIO.kCMIODeviceUnknown, CoreMediaIO.kCMIOObjectUnknown)

        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_Unknown, fourcc(b"unkn"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DV_NTSC, fourcc(b"dvc "))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DV_PAL, fourcc(b"dvcp"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro_NTSC, fourcc(b"dvpn"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro_PAL, fourcc(b"dvpp"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro50_NTSC, fourcc(b"dv5n"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro50_PAL, fourcc(b"dv5p"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro100_NTSC, fourcc(b"dv1n"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro100_PAL, fourcc(b"dv1p"))
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_DVCPro100_720p, fourcc(b"dvhp"))
        self.assertEqual(
            CoreMediaIO.kCMIOAVCDeviceType_DVCProHD_1080i50, fourcc(b"dvh5")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOAVCDeviceType_DVCProHD_1080i60, fourcc(b"dvh6")
        )
        self.assertEqual(CoreMediaIO.kCMIOAVCDeviceType_MPEG2, fourcc(b"mpg2"))

        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeSD525_60, 0x00)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeSDL525_60, 0x04)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHD1125_60, 0x08)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeSD625_50, 0x80)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeSDL625_50, 0x84)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHD1250_50, 0x88)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMPEG25Mbps_60, 0x10)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHDV1_60, 0x10)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMPEG12Mbps_60, 0x14)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMPEG6Mbps_60, 0x18)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMPEG25Mbps_50, 0x90)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHDV1_50, 0x90)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMPEG12Mbps_50, 0x94)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMPEG6Mbps_50, 0x98)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVHS, 0x01)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeVHSNTSC, 0x05)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeVHSMPAL, 0x25)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeVHSPAL, 0xA5)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeVHSNPAL, 0xB5)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeVHSSECAM, 0xC5)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeVHSMESECAM, 0xD5)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeSVHS525_60, 0x0D)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeSVHS625_50, 0xED)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalMode8mmNTSC, 0x06)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalMode8mmPAL, 0x86)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHi8NTSC, 0x0E)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHi8PAL, 0x8E)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMicroMV12Mbps_60, 0x24)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMicroMV6Mbps_60, 0x28)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMicroMV12Mbps_50, 0xA4)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeMicroMV6Mbps_50, 0xA8)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeAudio, 0x20)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHDV2_60, 0x1A)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeHDV2_50, 0x9A)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVCPro25_625_50, 0xF8)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVCPro25_525_60, 0x78)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVCPro50_625_50, 0xF4)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVCPro50_525_60, 0x74)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVCPro100_50, 0xF0)
        self.assertEqual(CoreMediaIO.kCMIODeviceAVCSignalModeDVCPro100_60, 0x70)

        self.assertEqual(CoreMediaIO.kCMIODevicePropertyPlugIn, fourcc(b"plug"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyDeviceUID, fourcc(b"uid "))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyModelUID, fourcc(b"muid"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyTransportType, fourcc(b"tran"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyDeviceIsAlive, fourcc(b"livn"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyDeviceHasChanged, fourcc(b"diff")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyDeviceIsRunning, fourcc(b"goin")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyDeviceIsRunningSomewhere, fourcc(b"gone")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyDeviceCanBeDefaultDevice, fourcc(b"dflt")
        )
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyHogMode, fourcc(b"oink"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyLatency, fourcc(b"ltnc"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyStreams, fourcc(b"stm#"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyStreamConfiguration, fourcc(b"slay")
        )
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyDeviceMaster, fourcc(b"pmnh"))
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyDeviceControl, fourcc(b"pmnh"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyExcludeNonDALAccess, fourcc(b"ixna")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyClientSyncDiscontinuity, fourcc(b"pmcs")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertySMPTETimeCallback, fourcc(b"pmsc")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyCanProcessAVCCommand, fourcc(b"pmac")
        )
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyAVCDeviceType, fourcc(b"pmat"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyAVCDeviceSignalMode, fourcc(b"pmsm")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyCanProcessRS422Command, fourcc(b"r422")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyLinkedCoreAudioDeviceUID, fourcc(b"plud")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyVideoDigitizerComponents, fourcc(b"vdig")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertySuspendedByUser, fourcc(b"sbyu")
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyLinkedAndSyncedCoreAudioDeviceUID,
            fourcc(b"plsd"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyIIDCInitialUnitSpace, fourcc(b"iuns")
        )
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyIIDCCSRData, fourcc(b"csrd"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyCanSwitchFrameRatesWithoutFrameDrops,
            fourcc(b"frnd"),
        )
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyLocation, fourcc(b"dloc"))
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyDeviceHasStreamingError, fourcc(b"serr")
        )

        self.assertEqual(CoreMediaIO.kCMIODevicePropertyLocationUnknown, 0)
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyLocationBuiltInDisplay, 1)
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyLocationExternalDisplay, 2)
        self.assertEqual(CoreMediaIO.kCMIODevicePropertyLocationExternalDevice, 3)
        self.assertEqual(
            CoreMediaIO.kCMIODevicePropertyLocationExternalWirelessDevice, 4
        )

    @min_os_level("10.7")
    def testFunctions(self):
        CoreMediaIO.CMIODeviceStartStream
        CoreMediaIO.CMIODeviceStopStream

        self.assertNotIsInstance(CoreMediaIO.CMIODeviceProcessAVCCommand, objc.function)
        self.assertNotIsInstance(
            CoreMediaIO.CMIODeviceProcessRS422Command, objc.function
        )



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, fourcc


class TestCMIOHardwareSystem(TestCase):
    def testConstants(self):
        self.assertEqual(CoreMediaIO.kCMIOSystemObjectClassID, fourcc(b"asys"))
        self.assertEqual(CoreMediaIO.kCMIOObjectSystemObject, 1)

        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyProcessIsMaster, fourcc(b"mast")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyProcessIsMain, fourcc(b"main")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyIsInitingOrExiting, fourcc(b"inot")
        )
        self.assertEqual(CoreMediaIO.kCMIOHardwarePropertyDevices, fourcc(b"dev#"))
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyDefaultInputDevice, fourcc(b"dIn ")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyDefaultOutputDevice, fourcc(b"dOut")
        )
        self.assertEqual(CoreMediaIO.kCMIOHardwarePropertyDeviceForUID, fourcc(b"duid"))
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertySleepingIsAllowed, fourcc(b"slep")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyUnloadingIsAllowed, fourcc(b"unld")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyPlugInForBundleID, fourcc(b"pibi")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyUserSessionIsActiveOrHeadless,
            fourcc(b"user"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertySuspendedBySystem, fourcc(b"sbys")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyAllowScreenCaptureDevices, fourcc(b"yes ")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOHardwarePropertyAllowWirelessScreenCaptureDevices,
            fourcc(b"wscd"),
        )



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, min_sdk_level, min_os_level


class TestCMIOExtensionStreamHelper(CoreMediaIO.NSObject):
    def streamPropertiesForProperties_error_(self, a, b):
        return 1

    def setStreamProperties_error_(self, a, b):
        return 1

    def authorizedToStartStreamForClient_(self, a):
        return 1

    def startStreamAndReturnError_(self, a):
        return 1

    def stopStreamAndReturnError_(self, a):
        return 1


class TestCMIOExtensionStream(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreMediaIO.CMIOExtensionStreamDirection)
        self.assertIsEnumType(CoreMediaIO.CMIOExtensionStreamClockType)
        self.assertIsEnumType(CoreMediaIO.CMIOExtensionStreamDiscontinuityFlags)

    def test_constants(self):
        self.assertEqual(CoreMediaIO.CMIOExtensionStreamDirectionSource, 0)
        self.assertEqual(CoreMediaIO.CMIOExtensionStreamDirectionSink, 1)

        self.assertEqual(CoreMediaIO.CMIOExtensionStreamClockTypeHostTime, 0)
        self.assertEqual(
            CoreMediaIO.CMIOExtensionStreamClockTypeLinkedCoreAudioDeviceUID, 1
        )
        self.assertEqual(CoreMediaIO.CMIOExtensionStreamClockTypeCustom, 2)

        self.assertEqual(CoreMediaIO.CMIOExtensionStreamDiscontinuityFlagNone, 0)
        self.assertEqual(
            CoreMediaIO.CMIOExtensionStreamDiscontinuityFlagUnknown, 1 << 0
        )
        self.assertEqual(CoreMediaIO.CMIOExtensionStreamDiscontinuityFlagTime, 1 << 1)
        self.assertEqual(
            CoreMediaIO.CMIOExtensionStreamDiscontinuityFlagSampleDropped, 1 << 6
        )

    @min_sdk_level("12.3")
    def test_protocols(self):
        self.assertProtocolExists("CMIOExtensionStreamSource")

    def test_methods(self):
        self.assertArgHasType(
            TestCMIOExtensionStreamHelper.streamPropertiesForProperties_error_,
            1,
            b"o^@",
        )

        self.assertResultIsBOOL(
            TestCMIOExtensionStreamHelper.setStreamProperties_error_
        )
        self.assertArgHasType(
            TestCMIOExtensionStreamHelper.setStreamProperties_error_, 1, b"o^@"
        )

        self.assertResultIsBOOL(
            TestCMIOExtensionStreamHelper.authorizedToStartStreamForClient_
        )

        self.assertResultIsBOOL(
            TestCMIOExtensionStreamHelper.startStreamAndReturnError_
        )
        self.assertArgHasType(
            TestCMIOExtensionStreamHelper.startStreamAndReturnError_, 0, b"o^@"
        )

        self.assertResultIsBOOL(TestCMIOExtensionStreamHelper.stopStreamAndReturnError_)
        self.assertArgHasType(
            TestCMIOExtensionStreamHelper.stopStreamAndReturnError_, 0, b"o^@"
        )

    @min_os_level("12.3")
    def test_methods12_3(self):
        self.assertArgIsBlock(
            CoreMediaIO.CMIOExtensionStream.consumeSampleBufferFromClient_completionHandler_,
            1,
            b"v^{opaqueCMSampleBuffer=}QIZ@",
        )



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCMIOExtensionProperties(TestCase):
    def test_enum_types(self):
        self.assertIsTypedEnum(CoreMediaIO.CMIOExtensionProperty, str)

    @min_os_level("12.3")
    def test_constants(self):
        self.assertIsInstance(CoreMediaIO.CMIOExtensionPropertyProviderName, str)
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyProviderManufacturer, str
        )
        self.assertIsInstance(CoreMediaIO.CMIOExtensionPropertyDeviceModel, str)
        self.assertIsInstance(CoreMediaIO.CMIOExtensionPropertyDeviceIsSuspended, str)
        self.assertIsInstance(CoreMediaIO.CMIOExtensionPropertyDeviceTransportType, str)
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyDeviceLinkedCoreAudioDeviceUID, str
        )
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyDeviceCanBeDefaultInputDevice, str
        )
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyDeviceCanBeDefaultOutputDevice, str
        )
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyStreamActiveFormatIndex, str
        )
        self.assertIsInstance(CoreMediaIO.CMIOExtensionPropertyStreamFrameDuration, str)
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyStreamMaxFrameDuration, str
        )
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyStreamSinkBufferQueueSize, str
        )
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyStreamSinkBuffersRequiredForStartup, str
        )
        self.assertIsInstance(
            CoreMediaIO.CMIOExtensionPropertyStreamSinkBufferUnderrunCount, str
        )
        self.assertIsInstance(CoreMediaIO.CMIOExtensionPropertyStreamSinkEndOfData, str)

    @min_os_level("12.3")
    def test_methods(self):
        self.assertArgIsBOOL(
            CoreMediaIO.CMIOExtensionPropertyAttributes.propertyAttributesWithMinValue_maxValue_validValues_readOnly_,
            3,
        )
        self.assertArgIsBOOL(
            CoreMediaIO.CMIOExtensionPropertyAttributes.initWithMinValue_maxValue_validValues_readOnly_,
            3,
        )
        self.assertResultIsBOOL(CoreMediaIO.CMIOExtensionPropertyAttributes.isReadOnly)



# This file is generated by objective.metadata
#
# Last update: Mon Feb 21 22:38:40 2022
#
# flake8: noqa

import objc, sys
from typing import NewType

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
misc.update(
    {
        "CMIODeviceRS422Command": objc.createStructType(
            "CoreMediaIO.CMIODeviceRS422Command",
            b"{CMIODeviceRS422Command=^CI^CII}",
            [
                "mCommand",
                "mCommandLength",
                "mResponse",
                "mResponseLength",
                "mResponseUsed",
            ],
        ),
        "CMIODeviceAVCCommand": objc.createStructType(
            "CoreMediaIO.CMIODeviceAVCCommand",
            b"{CMIODeviceAVCCommand=^CI^CII}",
            [
                "mCommand",
                "mCommandLength",
                "mResponse",
                "mResponseLength",
                "mResponseUsed",
            ],
        ),
        "CMIODeviceSMPTETimeCallback": objc.createStructType(
            "CoreMediaIO.CMIODeviceSMPTETimeCallback",
            b"{CMIODeviceSMPTETimeCallback=^?^v}",
            ["mGetSMPTETimeProc", "mRefCon"],
        ),
        "CMIOStreamScheduledOutputNotificationProcAndRefCon": objc.createStructType(
            "CoreMediaIO.CMIOStreamScheduledOutputNotificationProcAndRefCon",
            b"{CMIOStreamScheduledOutputNotificationProcAndRefCon=^?^?}",
            ["scheduledOutputNotificationProc", "scheduledOutputNotificationRefCon"],
        ),
        "CMIOStreamDeck": objc.createStructType(
            "CoreMediaIO.CMIOStreamDeck",
            b"{CMIOStreamDeck=III}",
            ["mStatus", "mState", "mState2"],
        ),
        "CMIODeviceStreamConfiguration": objc.createStructType(
            "CoreMediaIO.CMIODeviceStreamConfiguration",
            b"{CMIODeviceStreamConfiguration=I^I}",
            ["mNumberStreams", "mNumberChannels"],
        ),
        "CMIOObjectPropertyAddress": objc.createStructType(
            "CoreMediaIO.CMIOObjectPropertyAddress",
            b"{CMIOObjectPropertyAddress=III}",
            ["mSelector", "mScope", "mElement"],
        ),
    }
)
constants = """$CMIOExtensionInfoDictionaryKey$CMIOExtensionMachServiceNameKey$CMIOExtensionPropertyDeviceCanBeDefaultInputDevice$CMIOExtensionPropertyDeviceCanBeDefaultOutputDevice$CMIOExtensionPropertyDeviceIsSuspended$CMIOExtensionPropertyDeviceLinkedCoreAudioDeviceUID$CMIOExtensionPropertyDeviceModel$CMIOExtensionPropertyDeviceTransportType$CMIOExtensionPropertyProviderManufacturer$CMIOExtensionPropertyProviderName$CMIOExtensionPropertyStreamActiveFormatIndex$CMIOExtensionPropertyStreamFrameDuration$CMIOExtensionPropertyStreamMaxFrameDuration$CMIOExtensionPropertyStreamSinkBufferQueueSize$CMIOExtensionPropertyStreamSinkBufferUnderrunCount$CMIOExtensionPropertyStreamSinkBuffersRequiredForStartup$CMIOExtensionPropertyStreamSinkEndOfData$kCMIOBlockBufferAttachmentKey_CVPixelBufferReference$kCMIOSampleBufferAttachmentKey_CAAudioTimeStamp$kCMIOSampleBufferAttachmentKey_ClientSequenceID$kCMIOSampleBufferAttachmentKey_ClosedCaptionSampleBuffer$kCMIOSampleBufferAttachmentKey_DiscontinuityFlags$kCMIOSampleBufferAttachmentKey_HDV1_PackData$kCMIOSampleBufferAttachmentKey_HDV2_VAUX$kCMIOSampleBufferAttachmentKey_HostTime$kCMIOSampleBufferAttachmentKey_MouseAndKeyboardModifiers$kCMIOSampleBufferAttachmentKey_MuxedSourcePresentationTimeStamp$kCMIOSampleBufferAttachmentKey_NativeSMPTEFrameCount$kCMIOSampleBufferAttachmentKey_NoDataMarker$kCMIOSampleBufferAttachmentKey_NumberOfVideoFramesInBuffer$kCMIOSampleBufferAttachmentKey_NumberOfVideoFramesInGOP$kCMIOSampleBufferAttachmentKey_PulldownCadenceInfo$kCMIOSampleBufferAttachmentKey_RepeatedBufferContents$kCMIOSampleBufferAttachmentKey_SMPTETime$kCMIOSampleBufferAttachmentKey_SequenceNumber$kCMIOSampleBufferAttachmentKey_SourceAudioFormatDescription$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorFrameRect$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorIsDrawnInFramebuffer$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorIsVisible$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorPositionX$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorPositionY$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorReference$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorScale$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_CursorSeed$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_KeyboardModifiers$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_KeyboardModifiersEvent$kCMIOSampleBufferAttachment_MouseAndKeyboardModifiersKey_MouseButtonState$"""
enums = """$CMIOExtensionStreamClockTypeCustom@2$CMIOExtensionStreamClockTypeHostTime@0$CMIOExtensionStreamClockTypeLinkedCoreAudioDeviceUID@1$CMIOExtensionStreamDirectionSink@1$CMIOExtensionStreamDirectionSource@0$CMIOExtensionStreamDiscontinuityFlagNone@0$CMIOExtensionStreamDiscontinuityFlagSampleDropped@64$CMIOExtensionStreamDiscontinuityFlagTime@2$CMIOExtensionStreamDiscontinuityFlagUnknown@1$kCMIOAVCDeviceType_DVCPro100_720p@1685481584$kCMIOAVCDeviceType_DVCPro100_NTSC@1685467502$kCMIOAVCDeviceType_DVCPro100_PAL@1685467504$kCMIOAVCDeviceType_DVCPro50_NTSC@1685468526$kCMIOAVCDeviceType_DVCPro50_PAL@1685468528$kCMIOAVCDeviceType_DVCProHD_1080i50@1685481525$kCMIOAVCDeviceType_DVCProHD_1080i60@1685481526$kCMIOAVCDeviceType_DVCPro_NTSC@1685483630$kCMIOAVCDeviceType_DVCPro_PAL@1685483632$kCMIOAVCDeviceType_DV_NTSC@1685480224$kCMIOAVCDeviceType_DV_PAL@1685480304$kCMIOAVCDeviceType_MPEG2@1836082994$kCMIOAVCDeviceType_Unknown@1970170734$kCMIOBacklightCompensationControlClassID@1651207284$kCMIOBlackLevelControlClassID@1651207286$kCMIOBooleanControlClassID@1953458028$kCMIOBooleanControlPropertyValue@1650685548$kCMIOBrightnessControlClassID@1651665268$kCMIOContrastControlClassID@1668576116$kCMIOControlClassID@1633907820$kCMIOControlPropertyElement@1667591277$kCMIOControlPropertyScope@1668506480$kCMIOControlPropertyVariant@1668702578$kCMIODataDestinationControlClassID@1684370292$kCMIODataSourceControlClassID@1685287523$kCMIODeckShuttlePause@0$kCMIODeckShuttlePlay1x@6$kCMIODeckShuttlePlayFast@7$kCMIODeckShuttlePlayFaster@8$kCMIODeckShuttlePlayFastest@9$kCMIODeckShuttlePlayHighSpeed@10$kCMIODeckShuttlePlayNextFrame@1$kCMIODeckShuttlePlayPreviousFrame@-1$kCMIODeckShuttlePlaySlow1@3$kCMIODeckShuttlePlaySlow2@4$kCMIODeckShuttlePlaySlow3@5$kCMIODeckShuttlePlaySlowest@2$kCMIODeckShuttleReverse1x@-6$kCMIODeckShuttleReverseFast@-7$kCMIODeckShuttleReverseFaster@-8$kCMIODeckShuttleReverseFastest@-9$kCMIODeckShuttleReverseHighSpeed@-10$kCMIODeckShuttleReverseSlow1@-3$kCMIODeckShuttleReverseSlow2@-4$kCMIODeckShuttleReverseSlow3@-5$kCMIODeckShuttleReverseSlowest@-2$kCMIODeckStateFastForward@6$kCMIODeckStateFastRewind@7$kCMIODeckStatePause@2$kCMIODeckStatePlay@1$kCMIODeckStatePlayReverse@5$kCMIODeckStatePlaySlow@3$kCMIODeckStateReverseSlow@4$kCMIODeckStateStop@0$kCMIODeckStatusBusy@1$kCMIODeckStatusLocal@2$kCMIODeckStatusNoDevice@7$kCMIODeckStatusNotThreaded@3$kCMIODeckStatusOpcode@5$kCMIODeckStatusSearchingForDevice@6$kCMIODeckStatusTapeInserted@4$kCMIODeviceAVCSignalMode8mmNTSC@6$kCMIODeviceAVCSignalMode8mmPAL@134$kCMIODeviceAVCSignalModeAudio@32$kCMIODeviceAVCSignalModeDVCPro100_50@240$kCMIODeviceAVCSignalModeDVCPro100_60@112$kCMIODeviceAVCSignalModeDVCPro25_525_60@120$kCMIODeviceAVCSignalModeDVCPro25_625_50@248$kCMIODeviceAVCSignalModeDVCPro50_525_60@116$kCMIODeviceAVCSignalModeDVCPro50_625_50@244$kCMIODeviceAVCSignalModeDVHS@1$kCMIODeviceAVCSignalModeHD1125_60@8$kCMIODeviceAVCSignalModeHD1250_50@136$kCMIODeviceAVCSignalModeHDV1_50@144$kCMIODeviceAVCSignalModeHDV1_60@16$kCMIODeviceAVCSignalModeHDV2_50@154$kCMIODeviceAVCSignalModeHDV2_60@26$kCMIODeviceAVCSignalModeHi8NTSC@14$kCMIODeviceAVCSignalModeHi8PAL@142$kCMIODeviceAVCSignalModeMPEG12Mbps_50@148$kCMIODeviceAVCSignalModeMPEG12Mbps_60@20$kCMIODeviceAVCSignalModeMPEG25Mbps_50@144$kCMIODeviceAVCSignalModeMPEG25Mbps_60@16$kCMIODeviceAVCSignalModeMPEG6Mbps_50@152$kCMIODeviceAVCSignalModeMPEG6Mbps_60@24$kCMIODeviceAVCSignalModeMicroMV12Mbps_50@164$kCMIODeviceAVCSignalModeMicroMV12Mbps_60@36$kCMIODeviceAVCSignalModeMicroMV6Mbps_50@168$kCMIODeviceAVCSignalModeMicroMV6Mbps_60@40$kCMIODeviceAVCSignalModeSD525_60@0$kCMIODeviceAVCSignalModeSD625_50@128$kCMIODeviceAVCSignalModeSDL525_60@4$kCMIODeviceAVCSignalModeSDL625_50@132$kCMIODeviceAVCSignalModeSVHS525_60@13$kCMIODeviceAVCSignalModeSVHS625_50@237$kCMIODeviceAVCSignalModeVHSMESECAM@213$kCMIODeviceAVCSignalModeVHSMPAL@37$kCMIODeviceAVCSignalModeVHSNPAL@181$kCMIODeviceAVCSignalModeVHSNTSC@5$kCMIODeviceAVCSignalModeVHSPAL@165$kCMIODeviceAVCSignalModeVHSSECAM@197$kCMIODeviceClassID@1633969526$kCMIODevicePermissionsError@560492391$kCMIODevicePropertyAVCDeviceSignalMode@1886221165$kCMIODevicePropertyAVCDeviceType@1886216564$kCMIODevicePropertyCanProcessAVCCommand@1886216547$kCMIODevicePropertyCanProcessRS422Command@1916023346$kCMIODevicePropertyCanSwitchFrameRatesWithoutFrameDrops@1718775396$kCMIODevicePropertyClientSyncDiscontinuity@1886217075$kCMIODevicePropertyDeviceCanBeDefaultDevice@1684434036$kCMIODevicePropertyDeviceControl@1886219880$kCMIODevicePropertyDeviceHasChanged@1684629094$kCMIODevicePropertyDeviceHasStreamingError@1936028274$kCMIODevicePropertyDeviceIsAlive@1818850926$kCMIODevicePropertyDeviceIsRunning@1735354734$kCMIODevicePropertyDeviceIsRunningSomewhere@1735356005$kCMIODevicePropertyDeviceMaster@1886219880$kCMIODevicePropertyDeviceUID@1969841184$kCMIODevicePropertyExcludeNonDALAccess@1769500257$kCMIODevicePropertyHogMode@1869180523$kCMIODevicePropertyIIDCCSRData@1668510308$kCMIODevicePropertyIIDCInitialUnitSpace@1769303667$kCMIODevicePropertyLatency@1819569763$kCMIODevicePropertyLinkedAndSyncedCoreAudioDeviceUID@1886155620$kCMIODevicePropertyLinkedCoreAudioDeviceUID@1886156132$kCMIODevicePropertyLocation@1684828003$kCMIODevicePropertyLocationBuiltInDisplay@1$kCMIODevicePropertyLocationExternalDevice@3$kCMIODevicePropertyLocationExternalDisplay@2$kCMIODevicePropertyLocationExternalWirelessDevice@4$kCMIODevicePropertyLocationUnknown@0$kCMIODevicePropertyModelUID@1836411236$kCMIODevicePropertyPlugIn@1886156135$kCMIODevicePropertySMPTETimeCallback@1886221155$kCMIODevicePropertyScopeInput@1768845428$kCMIODevicePropertyScopeOutput@1869968496$kCMIODevicePropertyScopePlayThrough@1886679669$kCMIODevicePropertyStreamConfiguration@1936482681$kCMIODevicePropertyStreams@1937009955$kCMIODevicePropertySuspendedByUser@1935833461$kCMIODevicePropertyTransportType@1953653102$kCMIODevicePropertyVideoDigitizerComponents@1986292071$kCMIODeviceUnknown@0$kCMIODeviceUnsupportedFormatError@560226676$kCMIODirectionControlClassID@1684632165$kCMIOExposureControlClassID@2020635506$kCMIOExposureControlPropertyConvergenceSpeed@1701016432$kCMIOExposureControlPropertyIntegrationTime@1701408372$kCMIOExposureControlPropertyLockThreshold@1701602155$kCMIOExposureControlPropertyMaximumGain@1701667192$kCMIOExposureControlPropertyRegionOfInterest@1701998441$kCMIOExposureControlPropertyStability@1702065273$kCMIOExposureControlPropertyStable@1702065250$kCMIOExposureControlPropertyTarget@1702127476$kCMIOExposureControlPropertyUnlockThreshold@1702194283$kCMIOFeatureControlClassID@1718903668$kCMIOFeatureControlPropertyAbsoluteNative@1717792353$kCMIOFeatureControlPropertyAbsoluteRange@1717789042$kCMIOFeatureControlPropertyAbsoluteUnitName@1717794158$kCMIOFeatureControlPropertyAbsoluteValue@1717789046$kCMIOFeatureControlPropertyAutomaticManual@1717789037$kCMIOFeatureControlPropertyConvertAbsoluteToNative@1717645934$kCMIOFeatureControlPropertyConvertNativeToAbsolute@1718497889$kCMIOFeatureControlPropertyNativeData@1717792356$kCMIOFeatureControlPropertyNativeDataRange@1717789810$kCMIOFeatureControlPropertyNativeRange@1717792370$kCMIOFeatureControlPropertyNativeValue@1717792374$kCMIOFeatureControlPropertyOnOff@1717792623$kCMIOFeatureControlPropertyTune@1717793902$kCMIOFocusControlClassID@1717794163$kCMIOGainControlClassID@1734437230$kCMIOGammaControlClassID@1735224673$kCMIOHardwareBadDeviceError@560227702$kCMIOHardwareBadObjectError@560947818$kCMIOHardwareBadPropertySizeError@561211770$kCMIOHardwareBadStreamError@561214578$kCMIOHardwareIllegalOperationError@1852797029$kCMIOHardwareNoError@0$kCMIOHardwareNotRunningError@1937010544$kCMIOHardwareNotStoppedError@1920298528$kCMIOHardwarePropertyAllowScreenCaptureDevices@2036691744$kCMIOHardwarePropertyAllowWirelessScreenCaptureDevices@2004050788$kCMIOHardwarePropertyDefaultInputDevice@1682533920$kCMIOHardwarePropertyDefaultOutputDevice@1682929012$kCMIOHardwarePropertyDeviceForUID@1685416292$kCMIOHardwarePropertyDevices@1684370979$kCMIOHardwarePropertyIsInitingOrExiting@1768845172$kCMIOHardwarePropertyPlugInForBundleID@1885954665$kCMIOHardwarePropertyProcessIsMain@1835100526$kCMIOHardwarePropertyProcessIsMaster@1835103092$kCMIOHardwarePropertySleepingIsAllowed@1936483696$kCMIOHardwarePropertySuspendedBySystem@1935833459$kCMIOHardwarePropertyUnloadingIsAllowed@1970170980$kCMIOHardwarePropertyUserSessionIsActiveOrHeadless@1970496882$kCMIOHardwareSuspendedBySystemError@1684369017$kCMIOHardwareUnknownPropertyError@2003332927$kCMIOHardwareUnspecifiedError@2003329396$kCMIOHardwareUnsupportedOperationError@1970171760$kCMIOHueControlClassID@1752524064$kCMIOInvalidSequenceNumber@18446744073709551615$kCMIOIrisControlClassID@1769105779$kCMIOJackControlClassID@1784767339$kCMIONoiseReductionControlClassID@1932684914$kCMIOObjectClassID@1634689642$kCMIOObjectClassIDWildcard@707406378$kCMIOObjectPropertyClass@1668047219$kCMIOObjectPropertyCreator@1869638759$kCMIOObjectPropertyElementCategoryName@1818452846$kCMIOObjectPropertyElementMain@0$kCMIOObjectPropertyElementMaster@0$kCMIOObjectPropertyElementName@1818454126$kCMIOObjectPropertyElementNumberName@1818455662$kCMIOObjectPropertyElementWildcard@4294967295$kCMIOObjectPropertyListenerAdded@1818850145$kCMIOObjectPropertyListenerRemoved@1818850162$kCMIOObjectPropertyManufacturer@1819107691$kCMIOObjectPropertyName@1819173229$kCMIOObjectPropertyOwnedObjects@1870098020$kCMIOObjectPropertyOwner@1937007734$kCMIOObjectPropertyScopeGlobal@1735159650$kCMIOObjectPropertyScopeWildcard@707406378$kCMIOObjectPropertySelectorWildcard@707406378$kCMIOObjectSystemObject@1$kCMIOObjectUnknown@0$kCMIOOpticalFilterClassID@1869637236$kCMIOPanControlClassID@1885433376$kCMIOPanTiltAbsoluteControlClassID@1886675298$kCMIOPanTiltRelativeControlClassID@1886679660$kCMIOPlugInClassID@1634757735$kCMIOPlugInPropertyBundleID@1885956452$kCMIOPlugInPropertyIsExtension@1885956453$kCMIOPowerLineFrequencyControlClassID@1886873201$kCMIORollAbsoluteControlClassID@1919904865$kCMIOSampleBufferDiscontinuityFlag_BufferOverrun@128$kCMIOSampleBufferDiscontinuityFlag_ClientSyncDiscontinuity@1024$kCMIOSampleBufferDiscontinuityFlag_CodecSettingsChanged@131072$kCMIOSampleBufferDiscontinuityFlag_DataFormatChanged@8192$kCMIOSampleBufferDiscontinuityFlag_DataWasDropped@64$kCMIOSampleBufferDiscontinuityFlag_DataWasFlushed@32$kCMIOSampleBufferDiscontinuityFlag_DiscontinuityInDTS@256$kCMIOSampleBufferDiscontinuityFlag_DurationWasExtended@32768$kCMIOSampleBufferDiscontinuityFlag_MalformedData@16$kCMIOSampleBufferDiscontinuityFlag_NoDataMarker@4096$kCMIOSampleBufferDiscontinuityFlag_PacketError@4$kCMIOSampleBufferDiscontinuityFlag_RelatedToDiscontinuity@512$kCMIOSampleBufferDiscontinuityFlag_SleepWakeCycle@65536$kCMIOSampleBufferDiscontinuityFlag_StreamDiscontinuity@8$kCMIOSampleBufferDiscontinuityFlag_TimecodeDiscontinuity@2$kCMIOSampleBufferDiscontinuityFlag_TimingReferenceJumped@16384$kCMIOSampleBufferDiscontinuityFlag_TrickPlay@2048$kCMIOSampleBufferDiscontinuityFlag_UnknownDiscontinuity@1$kCMIOSampleBufferNoDataEvent_DeviceDidNotSync@2$kCMIOSampleBufferNoDataEvent_DeviceInWrongMode@3$kCMIOSampleBufferNoDataEvent_NoMedia@1$kCMIOSampleBufferNoDataEvent_ProcessingError@4$kCMIOSampleBufferNoDataEvent_SleepWakeCycle@5$kCMIOSampleBufferNoDataEvent_Unknown@0$kCMIOSampleBufferNoDiscontinuities@0$kCMIOSaturationControlClassID@1935766645$kCMIOSelectorControlClassID@1936483188$kCMIOSelectorControlPropertyAvailableItemNames@1935892846$kCMIOSelectorControlPropertyAvailableItems@1935892841$kCMIOSelectorControlPropertyCurrentItem@1935893353$kCMIOSelectorControlPropertyItemName@1935894894$kCMIOSharpnessControlClassID@1936224880$kCMIOShutterControlClassID@1936225394$kCMIOStreamClassID@1634956402$kCMIOStreamPropertyCanProcessDeckCommand@1885627236$kCMIOStreamPropertyClock@1886217068$kCMIOStreamPropertyDeck@1684366187$kCMIOStreamPropertyDeckCueing@1668638051$kCMIOStreamPropertyDeckDropness@1685221232$kCMIOStreamPropertyDeckFrameNumber@1952673636$kCMIOStreamPropertyDeckLocal@1819239276$kCMIOStreamPropertyDeckThreaded@1953002084$kCMIOStreamPropertyDeviceSyncTimeoutInMSec@1886219826$kCMIOStreamPropertyDirection@1935960434$kCMIOStreamPropertyEndOfData@1886217572$kCMIOStreamPropertyFirstOutputPresentationTimeStamp@1886351476$kCMIOStreamPropertyFormatDescription@1885762592$kCMIOStreamPropertyFormatDescriptions@1885762657$kCMIOStreamPropertyFrameRate@1852207732$kCMIOStreamPropertyFrameRateRanges@1718776423$kCMIOStreamPropertyFrameRates@1852207651$kCMIOStreamPropertyInitialPresentationTimeStampForLinkedAndSyncedAudio@1768975475$kCMIOStreamPropertyLatency@1819569763$kCMIOStreamPropertyMinimumFrameRate@1835430516$kCMIOStreamPropertyNoDataEventCount@1886219827$kCMIOStreamPropertyNoDataTimeoutInMSec@1886219825$kCMIOStreamPropertyOutputBufferQueueSize@1886220145$kCMIOStreamPropertyOutputBufferRepeatCount@1886220146$kCMIOStreamPropertyOutputBufferUnderrunCount@1886220149$kCMIOStreamPropertyOutputBuffersNeededForThrottledPlayback@1835624038$kCMIOStreamPropertyOutputBuffersRequiredForStartup@1886220147$kCMIOStreamPropertyPreferredFormatDescription@1886545508$kCMIOStreamPropertyPreferredFrameRate@1886545522$kCMIOStreamPropertyScheduledOutputNotificationProc@1936682608$kCMIOStreamPropertyStartingChannel@1935894638$kCMIOStreamPropertyStillImage@1937010023$kCMIOStreamPropertyStillImageFormatDescriptions@1937008244$kCMIOStreamPropertyTerminalType@1952805485$kCMIOStreamUnknown@0$kCMIOSystemObjectClassID@1634957683$kCMIOTemperatureControlClassID@1952804208$kCMIOTiltControlClassID@1953066100$kCMIOWhiteBalanceControlClassID@2003329644$kCMIOWhiteBalanceUControlClassID@2003329653$kCMIOWhiteBalanceVControlClassID@2003329654$kCMIOWhiteLevelControlClassID@2003332214$kCMIOZoomControlClassID@2054123373$kCMIOZoomRelativeControlClassID@2054122866$"""
misc.update(
    {
        "CMIOExtensionStreamDirection": NewType("CMIOExtensionStreamDirection", int),
        "CMIOExtensionStreamClockType": NewType("CMIOExtensionStreamClockType", int),
        "CMIOExtensionStreamDiscontinuityFlags": NewType(
            "CMIOExtensionStreamDiscontinuityFlags", int
        ),
    }
)
misc.update({"CMIOExtensionProperty": NewType("CMIOExtensionProperty", str)})
misc.update({})
functions = {
    "CMIOObjectShow": (b"vI",),
    "CMIOStreamDeckJog": (b"iIi",),
    "CMIOObjectAddPropertyListenerBlock": (
        b"iI^{CMIOObjectPropertyAddress=III}@@?",
        "",
        {
            "arguments": {
                1: {"type_modifier": "n"},
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "I"},
                            2: {
                                "type": "^{CMIOObjectPropertyAddress=III}",
                                "type_modifier": "n",
                                "c_array_length_in_arg": 1,
                            },
                        },
                    }
                },
            }
        },
    ),
    "CMIOStreamDeckCueTo": (b"iIQZ",),
    "CMIOObjectGetPropertyData": (
        b"iI^{CMIOObjectPropertyAddress=III}I^vI^I^v",
        "",
        {
            "arguments": {
                1: {"type_modifier": "n"},
                3: {"c_array_length_in_arg": 2, "type_modifier": "o"},
                5: {"type_modifier": "o"},
                6: {"c_array_length_in_arg": (4, 5), "type_modifier": "o"},
            }
        },
    ),
    "CMIOSampleBufferCreateForImageBuffer": (
        b"i^{__CFAllocator=}^{__CVBuffer=}^{opaqueCMFormatDescription=}^{_CMSampleTimingInfo={_CMTime=qiIq}{_CMTime=qiIq}{_CMTime=qiIq}}QI^^{opaqueCMSampleBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                3: {"type_modifier": "n"},
                6: {"already_cfretained": True, "type_modifier": "o"},
            },
        },
    ),
    "CMIOObjectRemovePropertyListener": (
        b"iI^{CMIOObjectPropertyAddress=III}^?^v",
        "",
        {
            "arguments": {
                1: {"type_modifier": "n"},
                2: {
                    "callable": {
                        "retval": {"type": b"i"},
                        "arguments": {
                            0: {"type": b"I"},
                            1: {"type": b"I"},
                            2: {"type": b"^{CMIOObjectPropertyAddress=III}"},
                            3: {"type": b"^v"},
                        },
                    }
                },
            }
        },
    ),
    "CMIOSampleBufferCreateNoDataMarker": (
        b"i^{__CFAllocator=}I^{opaqueCMFormatDescription=}QI^^{opaqueCMSampleBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {5: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CMIOObjectGetPropertyDataSize": (
        b"iI^{CMIOObjectPropertyAddress=III}I^v^I",
        "",
        {
            "arguments": {
                1: {"type_modifier": "n"},
                3: {"c_array_length_in_arg": 2, "type_modifier": "n"},
                4: {"type_modifier": "o"},
            }
        },
    ),
    "CMIOStreamClockPostTimingEvent": (b"i{_CMTime=qiIq}QZ@",),
    "CMIODeviceStopStream": (b"iII",),
    "CMIODeviceStartStream": (b"iII",),
    "CMIOSampleBufferCreate": (
        b"i^{__CFAllocator=}^{OpaqueCMBlockBuffer=}^{opaqueCMFormatDescription=}II^{_CMSampleTimingInfo={_CMTime=qiIq}{_CMTime=qiIq}{_CMTime=qiIq}}I^QQI^^{opaqueCMSampleBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                10: {"already_cfretained": True, "type_modifier": "o"},
                5: {"c_array_length_in_arg": 4, "type_modifier": "n"},
                7: {"c_array_length_in_arg": 6, "type_modifier": "n"},
            },
        },
    ),
    "CMIOStreamCopyBufferQueue": (
        b"iI^?^v^^{opaqueCMSimpleQueue=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                1: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"I"},
                            1: {"type": b"^v"},
                            2: {"type": b"^v"},
                        },
                    }
                },
                3: {"already_cfretained": True, "type_modifier": "o"},
            },
        },
    ),
    "CMIOObjectAddPropertyListener": (
        b"iI^{CMIOObjectPropertyAddress=III}^?^v",
        "",
        {
            "arguments": {
                1: {"type_modifier": "n"},
                2: {
                    "callable": {
                        "retval": {"type": b"i"},
                        "arguments": {
                            0: {"type": b"I"},
                            1: {"type": b"I"},
                            2: {"type": b"^{CMIOObjectPropertyAddress=III}"},
                            3: {"type": b"^v"},
                        },
                    }
                },
            }
        },
    ),
    "CMIOObjectHasProperty": (
        b"ZI^{CMIOObjectPropertyAddress=III}",
        "",
        {"arguments": {1: {"type_modifier": "n"}}},
    ),
    "CMIOObjectIsPropertySettable": (
        b"iI^{CMIOObjectPropertyAddress=III}^Z",
        "",
        {"arguments": {1: {"type_modifier": "n"}, 2: {"type_modifier": "o"}}},
    ),
    "CMIOSampleBufferGetSequenceNumber": (b"Q^{opaqueCMSampleBuffer=}",),
    "CMIOStreamClockCreate": (
        b"i^{__CFAllocator=}^{__CFString=}^v{_CMTime=qiIq}II^@",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {6: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CMIOStreamClockInvalidate": (b"i@",),
    "CMIOObjectsPublishedAndDied": (
        b"i^^{CMIOHardwarePlugInInterface=^v^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?}II^II^I",
    ),
    "CMIOObjectSetPropertyData": (b"iI^{CMIOObjectPropertyAddress=III}I^vI^v",),
    "CMIOSampleBufferGetDiscontinuityFlags": (b"I^{opaqueCMSampleBuffer=}",),
    "CMIOStreamDeckPlay": (b"iI",),
    "CMIOSampleBufferSetDiscontinuityFlags": (
        b"v^{__CFAllocator=}^{opaqueCMSampleBuffer=}I",
    ),
    "CMIOStreamDeckStop": (b"iI",),
    "CMIOObjectPropertiesChanged": (
        b"i^^{CMIOHardwarePlugInInterface=^v^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?}II^{CMIOObjectPropertyAddress=III}",
    ),
    "CMIOSampleBufferSetSequenceNumber": (
        b"v^{__CFAllocator=}^{opaqueCMSampleBuffer=}Q",
    ),
    "CMIOSampleBufferCopySampleAttachments": (
        b"i^{opaqueCMSampleBuffer=}^{opaqueCMSampleBuffer=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CMIOObjectCreate": (
        b"i^^{CMIOHardwarePlugInInterface=^v^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?^?}II^I",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CMIOStreamClockConvertHostTimeToDeviceTime": (b"{_CMTime=qiIq}Q@",),
    "CMIOObjectRemovePropertyListenerBlock": (
        b"iI^{CMIOObjectPropertyAddress=III}@@?",
        "",
        {
            "arguments": {
                1: {"type_modifier": "n"},
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "I"},
                            2: {
                                "type": "^{CMIOObjectPropertyAddress=III}",
                                "type_modifier": "n",
                                "c_array_length_in_arg": 1,
                            },
                        },
                    }
                },
            }
        },
    ),
    "CMIOSampleBufferCopyNonRequiredAttachments": (
        b"i^{opaqueCMSampleBuffer=}^{opaqueCMSampleBuffer=}I",
        "",
        {"retval": {"already_cfretained": True}},
    ),
}
aliases = {
    "kCMIODevicePropertyDeviceMaster": "kCMIODevicePropertyDeviceControl",
    "kCMIODeviceUnknown": "kCMIOObjectUnknown",
    "kCMIOObjectPropertyElementMaster": "kCMIOObjectPropertyElementMain",
    "kCMIOStreamUnknown": "kCMIOObjectUnknown",
}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"CMIOExtensionDevice",
        b"addStream:error:",
        {"retval": {"type": b"Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"CMIOExtensionDevice",
        b"removeStream:error:",
        {"retval": {"type": b"Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"CMIOExtensionPropertyAttributes",
        b"initWithMinValue:maxValue:validValues:readOnly:",
        {"arguments": {5: {"type": b"Z"}}},
    )
    r(b"CMIOExtensionPropertyAttributes", b"isReadOnly", {"retval": {"type": b"Z"}})
    r(
        b"CMIOExtensionPropertyAttributes",
        b"propertyAttributesWithMinValue:maxValue:validValues:readOnly:",
        {"arguments": {5: {"type": b"Z"}}},
    )
    r(
        b"CMIOExtensionProvider",
        b"addDevice:error:",
        {"retval": {"type": b"Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"CMIOExtensionProvider",
        b"removeDevice:error:",
        {"retval": {"type": b"Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"CMIOExtensionStream",
        b"consumeSampleBufferFromClient:completionHandler:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"^{opaqueCMSampleBuffer=}"},
                            2: {"type": b"Q"},
                            3: {"type": b"I"},
                            4: {"type": b"Z"},
                            5: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"CMIOExtensionStreamCustomClockConfiguration",
        b"customClockConfigurationWithClockName:sourceIdentifier:getTimeCallMinimumInterval:numberOfEventsForRateSmoothing:numberOfAveragesForRateSmoothing:",
        {"arguments": {4: {"type": b"{_CMTime=qiIq}"}}},
    )
    r(
        b"CMIOExtensionStreamCustomClockConfiguration",
        b"getTimeCallMinimumInterval",
        {"retval": {"type": b"{_CMTime=qiIq}"}},
    )
    r(
        b"CMIOExtensionStreamCustomClockConfiguration",
        b"initWithClockName:sourceIdentifier:getTimeCallMinimumInterval:numberOfEventsForRateSmoothing:numberOfAveragesForRateSmoothing:",
        {"arguments": {4: {"type": b"{_CMTime=qiIq}"}}},
    )
    r(
        b"CMIOExtensionStreamFormat",
        b"initWithFormatDescription:maxFrameDuration:minFrameDuration:validFrameDurations:",
        {"arguments": {3: {"type": b"{_CMTime=qiIq}"}, 4: {"type": b"{_CMTime=qiIq}"}}},
    )
    r(
        b"CMIOExtensionStreamFormat",
        b"maxFrameDuration",
        {"retval": {"type": b"{_CMTime=qiIq}"}},
    )
    r(
        b"CMIOExtensionStreamFormat",
        b"minFrameDuration",
        {"retval": {"type": b"{_CMTime=qiIq}"}},
    )
    r(
        b"CMIOExtensionStreamFormat",
        b"streamFormatWithFormatDescription:maxFrameDuration:minFrameDuration:validFrameDurations:",
        {"arguments": {3: {"type": b"{_CMTime=qiIq}"}, 4: {"type": b"{_CMTime=qiIq}"}}},
    )
    r(
        b"NSObject",
        b"authorizedToStartStreamForClient:",
        {"required": True, "retval": {"type": b"Z"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"availableProperties", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"connectClient:error:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"devicePropertiesForProperties:error:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"disconnectClient:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"formats", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"providerPropertiesForProperties:error:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"setDeviceProperties:error:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"setProviderProperties:error:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"setStreamProperties:error:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"startStreamAndReturnError:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"stopStreamAndReturnError:",
        {
            "required": True,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"streamPropertiesForProperties:error:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"^@", "type_modifier": b"o"}},
        },
    )
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



import CoreMediaIO
from PyObjCTools.TestSupport import TestCase, fourcc


class TestCMIOHardwareControl(TestCase):
    def testConstants(self):
        self.assertEqual(CoreMediaIO.kCMIOControlClassID, fourcc(b"actl"))
        self.assertEqual(CoreMediaIO.kCMIOBooleanControlClassID, fourcc(b"togl"))
        self.assertEqual(CoreMediaIO.kCMIOSelectorControlClassID, fourcc(b"slct"))
        self.assertEqual(CoreMediaIO.kCMIOFeatureControlClassID, fourcc(b"ftct"))

        self.assertEqual(CoreMediaIO.kCMIOJackControlClassID, fourcc(b"jack"))
        self.assertEqual(CoreMediaIO.kCMIODirectionControlClassID, fourcc(b"dire"))

        self.assertEqual(CoreMediaIO.kCMIODataSourceControlClassID, fourcc(b"dsrc"))
        self.assertEqual(
            CoreMediaIO.kCMIODataDestinationControlClassID, fourcc(b"dest")
        )

        self.assertEqual(CoreMediaIO.kCMIOBlackLevelControlClassID, fourcc(b"bklv"))
        self.assertEqual(CoreMediaIO.kCMIOWhiteLevelControlClassID, fourcc(b"whlv"))
        self.assertEqual(CoreMediaIO.kCMIOHueControlClassID, fourcc(b"hue "))
        self.assertEqual(CoreMediaIO.kCMIOSaturationControlClassID, fourcc(b"satu"))
        self.assertEqual(CoreMediaIO.kCMIOContrastControlClassID, fourcc(b"ctst"))
        self.assertEqual(CoreMediaIO.kCMIOSharpnessControlClassID, fourcc(b"shrp"))
        self.assertEqual(CoreMediaIO.kCMIOBrightnessControlClassID, fourcc(b"brit"))
        self.assertEqual(CoreMediaIO.kCMIOGainControlClassID, fourcc(b"gain"))
        self.assertEqual(CoreMediaIO.kCMIOIrisControlClassID, fourcc(b"iris"))
        self.assertEqual(CoreMediaIO.kCMIOShutterControlClassID, fourcc(b"shtr"))
        self.assertEqual(CoreMediaIO.kCMIOExposureControlClassID, fourcc(b"xpsr"))
        self.assertEqual(CoreMediaIO.kCMIOWhiteBalanceUControlClassID, fourcc(b"whbu"))
        self.assertEqual(CoreMediaIO.kCMIOWhiteBalanceVControlClassID, fourcc(b"whbv"))
        self.assertEqual(CoreMediaIO.kCMIOWhiteBalanceControlClassID, fourcc(b"whbl"))
        self.assertEqual(CoreMediaIO.kCMIOGammaControlClassID, fourcc(b"gmma"))
        self.assertEqual(CoreMediaIO.kCMIOTemperatureControlClassID, fourcc(b"temp"))
        self.assertEqual(CoreMediaIO.kCMIOZoomControlClassID, fourcc(b"zoom"))
        self.assertEqual(CoreMediaIO.kCMIOFocusControlClassID, fourcc(b"fcus"))
        self.assertEqual(CoreMediaIO.kCMIOPanControlClassID, fourcc(b"pan "))
        self.assertEqual(CoreMediaIO.kCMIOTiltControlClassID, fourcc(b"tilt"))
        self.assertEqual(CoreMediaIO.kCMIOOpticalFilterClassID, fourcc(b"opft"))
        self.assertEqual(
            CoreMediaIO.kCMIOBacklightCompensationControlClassID, fourcc(b"bklt")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOPowerLineFrequencyControlClassID, fourcc(b"pwfq")
        )
        self.assertEqual(CoreMediaIO.kCMIONoiseReductionControlClassID, fourcc(b"s2nr"))
        self.assertEqual(
            CoreMediaIO.kCMIOPanTiltAbsoluteControlClassID, fourcc(b"ptab")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOPanTiltRelativeControlClassID, fourcc(b"ptrl")
        )
        self.assertEqual(CoreMediaIO.kCMIOZoomRelativeControlClassID, fourcc(b"zomr"))
        self.assertEqual(CoreMediaIO.kCMIORollAbsoluteControlClassID, fourcc(b"rola"))

        self.assertEqual(CoreMediaIO.kCMIOControlPropertyScope, fourcc(b"cscp"))
        self.assertEqual(CoreMediaIO.kCMIOControlPropertyElement, fourcc(b"celm"))
        self.assertEqual(CoreMediaIO.kCMIOControlPropertyVariant, fourcc(b"cvar"))

        self.assertEqual(CoreMediaIO.kCMIOBooleanControlPropertyValue, fourcc(b"bcvl"))

        self.assertEqual(
            CoreMediaIO.kCMIOSelectorControlPropertyCurrentItem, fourcc(b"scci")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSelectorControlPropertyAvailableItems, fourcc(b"scai")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSelectorControlPropertyItemName, fourcc(b"scin")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOSelectorControlPropertyAvailableItemNames, fourcc(b"scan")
        )

        self.assertEqual(CoreMediaIO.kCMIOFeatureControlPropertyOnOff, fourcc(b"fcoo"))
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyAutomaticManual, fourcc(b"fcam")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyAbsoluteNative, fourcc(b"fcna")
        )
        self.assertEqual(CoreMediaIO.kCMIOFeatureControlPropertyTune, fourcc(b"fctn"))
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyNativeValue, fourcc(b"fcnv")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyAbsoluteValue, fourcc(b"fcav")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyNativeRange, fourcc(b"fcnr")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyAbsoluteRange, fourcc(b"fcar")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyConvertNativeToAbsolute,
            fourcc(b"fn2a"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyConvertAbsoluteToNative,
            fourcc(b"fa2n"),
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyAbsoluteUnitName, fourcc(b"fcun")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyNativeData, fourcc(b"fcnd")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOFeatureControlPropertyNativeDataRange, fourcc(b"fcdr")
        )

        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyRegionOfInterest, fourcc(b"eroi")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyLockThreshold, fourcc(b"elck")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyUnlockThreshold, fourcc(b"eulk")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyTarget, fourcc(b"etgt")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyConvergenceSpeed, fourcc(b"ecsp")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyStability, fourcc(b"esty")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyStable, fourcc(b"estb")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyIntegrationTime, fourcc(b"eint")
        )
        self.assertEqual(
            CoreMediaIO.kCMIOExposureControlPropertyMaximumGain, fourcc(b"emax")
        )
